<?php


class DPD_Calculator extends WPRuby_Abstract_Calculator
{
	public $settings;

	public function __construct($settings)
	{
		$this->settings = $settings;
	}


	/**
	 * @return array
	 */
	public function calculate($cart)
	{

		$packages = $this->get_package_details_by_boxpacker($cart);

		if (!is_array($packages)) {
			return [];
		}

		return $this->get_price_for_packages($packages, $cart);
	}


	private function get_price_for_packages($packages, $cart)
	{

		$parcels = [];
		foreach ($packages as $package) {
			$parcels[] = [
				"Weight" => $package['weight'],
				"Length" => $package['length'],
				"Width" =>  $package['width'],
				"Height" => $package['height'],
			];
		}

		$countries = include ROYALMAILPRO_DIR . 'shared/data/countries.php';
		$countryName = WC()->countries->get_countries()[$cart['destination']['country']];
		$normalizedCountryName = $this->normalizeCountryName($countryName);

		if (!isset($countries[$normalizedCountryName])) {
			return [];
		}

		$body = [
			'CollectionCountry' => 219, // UK
			'CollectionPostcode' => '',
			'CollectionTown' => '',
			'DeliveryCountry' => intval($countries[$normalizedCountryName]),
			'DeliveryPostcode' => $cart['destination']['postcode'],
			'DeliveryTown' => $cart['destination']['city'],
			'Parcels' => $parcels,
			"QuoteType" => "",
			"MultiDeliveryDestinations" => false,
			"IsBusinessRecipient" => false,
			"QuoteModified" => false,
			"QuoteRequestType" => "Default",
			"ConfigurationProfile" => "DPD.UK.LIVE"

		];

		$args = [
			'headers' => ['Content-Type' => 'application/json'],
			'body' => json_encode($body),
			'method' => 'POST',
		];

		$response = wp_remote_post('https://www.dpdlocal-online.co.uk/quotes/api/results/all', $args);


		$responseBody = json_decode(wp_remote_retrieve_body($response), true);

		if (!( isset($responseBody['Quotes']) && is_array($responseBody['Quotes']) )) {
			return [];
		}

		$domestic_options = (!is_array($this->settings['domestic_options'])) ? [$this->settings['domestic_options'] ] : $this->settings['domestic_options'];
		$intl_options = (!is_array($this->settings['intl_options'])) ? [$this->settings['intl_options'] ] : $this->settings['intl_options'];

		$enabledServices = array_merge($domestic_options, $intl_options);

		$totalCover = 0;
		if ($this->settings['add_cover'] === 'yes') {
			$totalCover = $this->getTotalCoverFee($packages);
		}

		$rates = [];
		foreach ($responseBody['Quotes'] as $quote) {
			$serviceSlug = $quote['Service']['ServiceSlug'];
			if(in_array($serviceSlug, $enabledServices)) {
				$rates[] = [
					'id'    => $serviceSlug,
					'label' => sprintf('%s: %s', $this->settings['title'], $quote['Service']['HtmlName']),
					'cost'  => $quote['Total'] + $totalCover,
				];
			}

		}

		return $rates;

	}

	private function getTotalCoverFee($packages)
	{
		if (!is_array($packages)) {
			$total = 0;
		}

		foreach ($packages as $package) {
			$total += $this->calculateCoverFees($package['value']);
		}

		return $total;
	}

	/**
	 * @param float $value
	 * @return float
	 */
	private function calculateCoverFees($value)
	{
		if ($value <= 50) {
			return 0;
		}

		if ($value >= 5000 ) {
			return 247.50;
		}

		$valueToCover = $value - 50;

		return round($valueToCover * 0.05, 2);
	}

}
